#!/usr/bin/php 
<?php

lasso_init();
require_once('/usr/share/php/lasso/lasso.php');

try {

	date_default_timezone_set('Australia/Sydney');

	$sp_server  = get_sp_server();

	//$login_nonce - this would be what we put into the session as the SAMLID key
	list($login_nonce,$login_request) = get_login_url($sp_server);

	describe_url($login_request);
	exit;


} catch(LassoServerProviderNotFoundError $e) {
	exit(1);
} catch(Exception $e) {
	print "Exception " . get_class($e) . " : " . $e->getLine() . ':' . $e->getMessage() . "\n";
	exit(1);
}

lasso_shutdown();

function get_sp_server($id=1) {

	switch ($id) {
		case 1:
			$xml = 'sp-metadata.xml';
			$key = NULL;
			$crt = NULL;
			break;
	}

	$key = NULL;
	$cert = NULL;

	$sp_server = new LassoServer($xml,$key,NULL,$crt);
	$sp_server->addProvider(LASSO_PROVIDER_ROLE_IDP,'idp-metadata.xml',NULL);

	return $sp_server;
}

/**
 * @param LassoServer $sp_server The service provider
 * @param string $idp_id The "entityid" of the identity provider you want to connect to
 *
 * @returns array(string,string) Nonce, and url to redirect to
 */
function get_login_url($sp_server,$idp_id=NULL) {

	$login = new LassoLogin($sp_server);
	$login->signatureHint = LASSO_PROFILE_SIGNATURE_HINT_FORBID; //Otherwise it complains about a bad private key

	$login->initAuthnRequest($idp_id); //entityID of the IDP

	$policy                = new LassoSamlp2NameIDPolicy();
	$policy->format        = LASSO_SAML2_NAME_IDENTIFIER_FORMAT_UNSPECIFIED;
	$policy->allowCreate   = false;

	$context                       = new LassoSamlp2RequestedAuthnContext();
	$context->comparison           = LASSO_LIB_AUTHN_CONTEXT_COMPARISON_MINIMUM;
	$context->authncontextclassref = array(LASSO_SAML2_AUTHN_CONTEXT_PASSWORD_PROTECTED_TRANSPORT);

	$login->request->NameIDPolicy          = $policy;
	$login->request->requestedauthncontext = $context;

	$login->buildAuthnRequestMsg();

	return array($login->request->id,$login->msgUrl);
}

function describe_url($url) {

	$url = parse_url($url);

	$parts = array();
	foreach (explode('&',urldecode($url['query'])) as $p) {

		$t = explode('=',$p);
		$parts[$t[0]] = $t[1];

	}

	print pretty_xml(gzinflate(base64_decode($parts['SAMLRequest']))) . "\n";
	
}

function pretty_xml($strxml){
	$process = proc_open(
	           	'xmllint --format -',
	           	array(
	           		0=>array('pipe','r'),
	           		1=>array('pipe','w'),
	           		2=>array('pipe','w')
	           ),
	           $pipes
	);

	fwrite($pipes[0],$strxml instanceof SimpleXMLElement ? $strxml->asXML() : $strxml);
	fclose($pipes[0]);

	$pretty = fread($pipes[1],5000);

	fclose($pipes[1]);
	fclose($pipes[2]);

	return $pretty;
}
